@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -130,7 +130,7 @@ class ConfigurableFileMerger(AbstractPerFileMerger):
         """
         affected_files = self.affected_files
         if affected_files is None:
-            config = self.merger.this_tree.branch.get_config()
+            config = self.merger.this_branch.get_config()
             # Until bzr provides a better policy for caching the config, we
             # just add the part we're interested in to the params to avoid
             # reading the config files repeatedly (bazaar.conf, location.conf,
